DROP VIEW RS_ENABLED_VIEW;
DROP TABLE AUX_RULESET_RESOURCES;
DROP TABLESPACE RSRESTS;
DROP TABLE RULESET_RESOURCES;
DROP TABLE RULESET_PROPERTIES;
DROP TABLE RULEAPP_PROPERTIES;
DROP TABLE RULESETS;
DROP TABLE RULEAPPS;

CREATE TABLE RULEAPPS (
  ID INT GENERATED ALWAYS AS IDENTITY (START WITH 1, INCREMENT BY 1) NOT NULL,
  NAME VARCHAR(256) NOT NULL,
  MAJOR_VERSION INT NOT NULL,
  MINOR_VERSION INT NOT NULL,
  CREATION_DATE DEC(19,0) NOT NULL,
  DISPLAY_NAME VARCHAR(256),
  DESCRIPTION VARCHAR(256),
  CONSTRAINT RA_PK PRIMARY KEY (ID),
  CONSTRAINT RA_UNIQUE UNIQUE (NAME, MAJOR_VERSION, MINOR_VERSION)
);

CREATE UNIQUE INDEX RULEAPPS_IDXU1 ON RULEAPPS (ID);
CREATE UNIQUE INDEX RULEAPPS_IDXU2 ON RULEAPPS (NAME, MAJOR_VERSION, MINOR_VERSION) NOT PADDED;

CREATE TABLE RULESETS (
  ID INT GENERATED ALWAYS AS IDENTITY (START WITH 1, INCREMENT BY 1) NOT NULL,
  NAME VARCHAR(256) NOT NULL,
  MAJOR_VERSION INT NOT NULL,
  MINOR_VERSION INT NOT NULL,
  CREATION_DATE DEC(19,0) NOT NULL,
  DISPLAY_NAME VARCHAR(256),
  DESCRIPTION VARCHAR(256),
  RULEAPP_ID INT NOT NULL,
  CONSTRAINT RS_PK PRIMARY KEY (ID),
  CONSTRAINT RS_FK FOREIGN KEY (RULEAPP_ID) REFERENCES RULEAPPS (ID) ON DELETE CASCADE,
  CONSTRAINT RS_UNIQUE UNIQUE (RULEAPP_ID, NAME, MAJOR_VERSION, MINOR_VERSION)
);

CREATE UNIQUE INDEX RULESETS_IDXU1 ON RULESETS (ID);
CREATE UNIQUE INDEX RULESETS_IDXU2 ON RULESETS (RULEAPP_ID, NAME, MAJOR_VERSION, MINOR_VERSION) NOT PADDED;

CREATE TABLE RULEAPP_PROPERTIES (
  ID INT GENERATED ALWAYS AS IDENTITY (START WITH 1, INCREMENT BY 1) NOT NULL,
  RULEAPP_ID INT NOT NULL,
  NAME VARCHAR(256) NOT NULL,
  VALUE VARCHAR(2000) NOT NULL,
  CONSTRAINT RA_PROPS_PK PRIMARY KEY (ID),
  CONSTRAINT RA_PROPS_FK FOREIGN KEY (RULEAPP_ID) REFERENCES RULEAPPS (ID) ON DELETE CASCADE,
  CONSTRAINT RA_PROPS_UN UNIQUE (RULEAPP_ID, NAME)
);

CREATE UNIQUE INDEX RULEAPPP_IDXU1 ON RULEAPP_PROPERTIES (ID);
CREATE UNIQUE INDEX RULEAPPP_IDXU2 ON RULEAPP_PROPERTIES (RULEAPP_ID, NAME) NOT PADDED;

CREATE TABLE RULESET_PROPERTIES (
  ID INT GENERATED ALWAYS AS IDENTITY (START WITH 1, INCREMENT BY 1) NOT NULL,
  RULESET_ID INT NOT NULL,
  NAME VARCHAR(256) NOT NULL,
  VALUE VARCHAR(2000) NOT NULL,
  CONSTRAINT RS_PROPS_PK PRIMARY KEY (ID),
  CONSTRAINT RS_PROPS_FK FOREIGN KEY (RULESET_ID) REFERENCES RULESETS (ID) ON DELETE CASCADE,
  CONSTRAINT RS_PROPS_UN UNIQUE (RULESET_ID, NAME)
);

CREATE UNIQUE INDEX RULESETP_IDXU1 ON RULESET_PROPERTIES (ID);
CREATE UNIQUE INDEX RULESETP_IDXU2 ON RULESET_PROPERTIES (RULESET_ID, NAME) NOT PADDED;

CREATE TABLE RULESET_RESOURCES (
  ID INT GENERATED ALWAYS AS IDENTITY (START WITH 1, INCREMENT BY 1) NOT NULL,
  RULESET_ID INT NOT NULL,
  ARCHIVE BLOB(1024 M) NOT NULL,
  CONSTRAINT RS_RES_PK PRIMARY KEY (ID),
  CONSTRAINT RS_RES_FK FOREIGN KEY (RULESET_ID) REFERENCES RULESETS (ID) ON DELETE CASCADE
);

CREATE UNIQUE INDEX RULESETR_IDXU1 ON RULESET_RESOURCES (ID);
CREATE LOB TABLESPACE RSRESTS LOG NO;
CREATE AUX TABLE AUX_RULESET_RESOURCES IN RSRESTS STORES RULESET_RESOURCES COLUMN ARCHIVE;
CREATE UNIQUE INDEX AUX_RULESET_RESOURCES_UI1 ON AUX_RULESET_RESOURCES;

CREATE VIEW RS_ENABLED_VIEW AS
  SELECT RA.ID AS RA_ID, RA.NAME AS RA_NAME, RA.MAJOR_VERSION AS RA_MAJVERS, RA.MINOR_VERSION AS RA_MINVERS,
  		 RS.ID AS RS_ID, RS.NAME AS RS_NAME, RS.MAJOR_VERSION AS RS_MAJVERS, RS.MINOR_VERSION AS RS_MINVERS
  FROM RULEAPPS RA, RULESETS RS, RULESET_PROPERTIES P
  WHERE RS.RULEAPP_ID = RA.ID AND RS.ID = P.RULESET_ID
  AND P.NAME = 'ruleset.status' AND P.VALUE = 'enabled';
